﻿Imports Microsoft.Xna.Framework
Imports Microsoft.Xna.Framework.Graphics
Imports Microsoft.Xna.Framework.Audio
Imports Microsoft.Xna.Framework.Content
Imports Microsoft.Xna.Framework.Media
Imports Microsoft.Xna.Framework.Input
Imports Microsoft.Xna.Framework.GamerServices

Public NotInheritable Class collisiondetection2d

    Public Shared Property AdditionalRenderTargetForCollision() As RenderTarget2D
        Get
            Return m_AdditionalRenderTargetForCollision
        End Get
        Set(value As RenderTarget2D)
            m_AdditionalRenderTargetForCollision = value
        End Set
    End Property
    Private Shared m_AdditionalRenderTargetForCollision As RenderTarget2D

    ''' <summary>
    ''' Receives two lists of 4points representing a rectangle and checks whether the two
    ''' shapes overlap
    ''' </summary>
    ''' <param name="Rect1">The 4 points of the first rectangle</param>
    ''' <param name="Rect2">The 4 points of the second rectangle</param>
    ''' <returns>True if they overlap/False otherwise</returns>
    Public Shared Function BoundingRectangle(Rect1 As List(Of Vector2), Rect2 As List(Of Vector2)) As Boolean
        Dim Triangle11 As List(Of Vector2)
        Dim Triangle12 As List(Of Vector2)
        _createTrianglesFromRectangle(Rect1, Triangle11, Triangle12)
        For Each point As Vector2 In Rect2
            If _isPointInsideTriangle(Triangle11, point) Then
                Return True
            End If
            If _isPointInsideTriangle(Triangle12, point) Then
                Return True
            End If
        Next
        Dim Triangle21 As List(Of Vector2)
        Dim Triangle22 As List(Of Vector2)
        _createTrianglesFromRectangle(Rect1, Triangle21, Triangle22)
        For Each point As Vector2 In Rect1
            If _isPointInsideTriangle(Triangle21, point) Then
                Return True
            End If
            If _isPointInsideTriangle(Triangle22, point) Then
                Return True
            End If
        Next
        Return False
    End Function

    ''' <summary>
    ''' Checks wheteher two circles overlap
    ''' </summary>
    ''' <param name="x1">The x coordinate of the center of the first circle</param>
    ''' <param name="y1">The y coordinate of the center of the first circle</param>
    ''' <param name="radius1">The radius of the first circle</param>
    ''' <param name="x2">The x coordinate of the center of the second circle</param>
    ''' <param name="y2">The y coordinate of the center of the second circle</param>
    ''' <param name="radius2">The radius of the second circle</param>
    ''' <returns>True if they overlap/False otherwise</returns>
    Public Shared Function BoundingCircle(x1 As Integer, y1 As Integer, radius1 As Integer, x2 As Integer, y2 As Integer, radius2 As Integer) As Boolean
        Dim V1 As New Vector2(x1, y1)
        Dim V2 As New Vector2(x2, y2)

        Dim Distance As Vector2 = V1 - V2

        If Distance.Length() < radius1 + radius2 Then
            Return True
        End If

        Return False
    End Function

    ''' <summary>
    ''' Checks whether two triangles overlap
    ''' </summary>
    ''' <param name="p1">The list of the 3 points of the first triangle</param>
    ''' <param name="p2">The list of the 3 points of the second triangle</param>
    ''' <returns>True if they overlap/False otherwise</returns>
    Public Shared Function BoundingTriangles(p1 As List(Of Vector2), p2 As List(Of Vector2)) As Boolean
        For i As Integer = 0 To 2
            If _isPointInsideTriangle(p1, p2(i)) Then
                Return True
            End If
        Next

        For i As Integer = 0 To 2
            If _isPointInsideTriangle(p2, p1(i)) Then
                Return True
            End If
        Next
        Return False
    End Function

    ''' <summary>
    ''' Per pixel collision detection that supports rotation
    ''' </summary>
    ''' <param name="Texture1">The first sprite's texture</param>
    ''' <param name="Texture2">The second sprite's texture</param>
    ''' <param name="Pos1">The first sprite's position</param>
    ''' <param name="Pos2">The second sprite's position</param>
    ''' <param name="Orig1">The first sprite's origin of rotation (texture reference point)</param>
    ''' <param name="Orig2">The second sprite's origin of rotation (texture reference point)</param>
    ''' <param name="Rect1">The first sprite's bounding rectangle</param>
    ''' <param name="Rect2">The second sprite's bounding rectangle</param>
    ''' <param name="Theta1">The first sprite's rotation</param>
    ''' <param name="Theta2">The second sprite's rotation</param>
    ''' <param name="spriteBatch">The current Spriteatch</param>
    ''' <returns>True if they overlap/False otherwise</returns>

    Public Shared Function PerPixelWR(Texture1 As Texture2D, Texture2 As Texture2D, Pos1 As Vector2, Pos2 As Vector2, Orig1 As Vector2, Orig2 As Vector2, _
    Rect1 As List(Of Vector2), Rect2 As List(Of Vector2), Theta1 As Single, Theta2 As Single, spriteBatch As SpriteBatch) As Boolean

        If Not BoundingRectangle(Rect1, Rect2) Then
            Return False
        End If
        Dim TextureData1 As Color() = _getSingleSpriteAloneFromScene(spriteBatch, Texture1, Pos1, Orig1, Theta1, Rect1)
        Dim TextureData2 As Color() = _getSingleSpriteAloneFromScene(spriteBatch, Texture2, Pos2, Orig2, Theta2, Rect2)

        Dim Rectangle1 As Rectangle = _getBoundingRectangleOfRotatedRectangle(Rect1)
        Dim Rectangle2 As Rectangle = _getBoundingRectangleOfRotatedRectangle(Rect2)

        Dim top As Integer = Math.Max(Rectangle1.Top, Rectangle2.Top)
        Dim bottom As Integer = Math.Min(Rectangle1.Bottom, Rectangle2.Bottom)
        Dim left As Integer = Math.Max(Rectangle1.Left, Rectangle2.Left)
        Dim right As Integer = Math.Min(Rectangle1.Right, Rectangle2.Right)

        For y As Integer = top To bottom - 1
            For x As Integer = left To right - 1
                Dim colorA As Color = TextureData1((x - Rectangle1.Left) + (y - Rectangle1.Top) * Rectangle1.Width)
                Dim colorB As Color = TextureData2((x - Rectangle2.Left) + (y - Rectangle2.Top) * Rectangle2.Width)
                'If colorA <> Color.White AndAlso colorB <> Color.White Then
                If colorA <> New Color(0, 0, 0, 0) AndAlso colorB <> New Color(0, 0, 0, 0) Then
                    Return True
                End If
                'End If

            Next
        Next

        Return False
    End Function

    Private Shared Function _getSingleSpriteAloneFromScene(spriteBatch As SpriteBatch, textureToDraw As Texture2D, Position As Vector2, Origin As Vector2, Theta As Single, RectanglePoints As List(Of Vector2)) As Color()
        spriteBatch.GraphicsDevice.SetRenderTarget(m_AdditionalRenderTargetForCollision)
        spriteBatch.GraphicsDevice.Clear(Color.Transparent)
        spriteBatch.Begin()
        spriteBatch.Draw(textureToDraw, Position, Nothing, Color.White, Theta, Origin, 1.0F, SpriteEffects.None, 0)
        spriteBatch.End()
        spriteBatch.GraphicsDevice.SetRenderTarget(Nothing, Nothing)
        Dim SceneTexture As Texture2D = m_AdditionalRenderTargetForCollision
        Dim BoundingRectangle As Rectangle = _getBoundingRectangleOfRotatedRectangle(RectanglePoints)
        Dim PixelSize As Integer = BoundingRectangle.Width * BoundingRectangle.Height
        Dim TextureData As Color() = New Color(PixelSize - 1) {}
        SceneTexture.GetData(0, BoundingRectangle, TextureData, 0, PixelSize)

        Return TextureData
    End Function

    Private Shared Function _isPointInsideTriangle(TrianglePoints As List(Of Vector2), p As Vector2) As Boolean
        ' Translated to C# from: http://www.ddj.com/184404201
        Dim e0 As Vector2 = p - TrianglePoints(0)
        Dim e1 As Vector2 = TrianglePoints(1) - TrianglePoints(0)
        Dim e2 As Vector2 = TrianglePoints(2) - TrianglePoints(0)

        Dim u As Single, v As Single = 0
        If e1.X = 0 Then
            If e2.X = 0 Then
                Return False
            End If
            u = e0.X / e2.X
            If u < 0 OrElse u > 1 Then
                Return False
            End If
            If e1.Y = 0 Then
                Return False
            End If
            v = (e0.Y - e2.Y * u) / e1.Y
            If v < 0 Then
                Return False
            End If
        Else
            Dim d As Single = e2.Y * e1.X - e2.X * e1.Y
            If d = 0 Then
                Return False
            End If
            u = (e0.Y * e1.X - e0.X * e1.Y) / d
            If u < 0 OrElse u > 1 Then
                Return False
            End If
            v = (e0.X - e2.X * u) / e1.X
            If v < 0 Then
                Return False
            End If
            If (u + v) > 1 Then
                Return False
            End If
        End If

        Return True
    End Function

    'AABB (axis aligned bounding box)
    Private Shared Function _getBoundingRectangleOfRotatedRectangle(RectanglePoints As List(Of Vector2)) As Rectangle
        Dim BoundingRectangleStart As New Vector2()
        BoundingRectangleStart.X = _getMinimumOf(RectanglePoints(0).X, RectanglePoints(1).X, RectanglePoints(2).X, RectanglePoints(3).X)
        BoundingRectangleStart.Y = _getMinimumOf(RectanglePoints(0).Y, RectanglePoints(1).Y, RectanglePoints(2).Y, RectanglePoints(3).Y)

        Dim BoundingRectangleWidth As Integer = -CInt(BoundingRectangleStart.X) + _getMaximumOf(RectanglePoints(0).X, RectanglePoints(1).X, RectanglePoints(2).X, RectanglePoints(3).X)
        Dim BoundingRectangleHeight As Integer = -CInt(BoundingRectangleStart.Y) + _getMaximumOf(RectanglePoints(0).Y, RectanglePoints(1).Y, RectanglePoints(2).Y, RectanglePoints(3).Y)

        Return New Rectangle(CInt(BoundingRectangleStart.X), CInt(BoundingRectangleStart.Y), BoundingRectangleWidth, BoundingRectangleHeight)

    End Function
    Private Shared Function _getMinimumOf(a1 As Single, a2 As Single, a3 As Single, a4 As Single) As Integer
        Return CInt(MathHelper.Min(MathHelper.Min(MathHelper.Min(a1, a2), a3), a4))
    End Function
    Private Shared Function _getMaximumOf(a1 As Single, a2 As Single, a3 As Single, a4 As Single) As Integer
        Return CInt(MathHelper.Max(MathHelper.Max(MathHelper.Max(a1, a2), a3), a4))
    End Function
    Private Shared Sub _createTrianglesFromRectangle(RectPoints As List(Of Vector2), ByRef Triangle1 As List(Of Vector2), ByRef Triangle2 As List(Of Vector2))
        Triangle1 = New List(Of Vector2)() From { _
            New Vector2(RectPoints(0).X, RectPoints(0).Y), _
            New Vector2(RectPoints(1).X, RectPoints(1).Y), _
            New Vector2(RectPoints(3).X, RectPoints(3).Y) _
        }
        Triangle2 = New List(Of Vector2)() From { _
            New Vector2(RectPoints(1).X, RectPoints(1).Y), _
            New Vector2(RectPoints(2).X, RectPoints(2).Y), _
            New Vector2(RectPoints(3).X, RectPoints(3).Y) _
        }
    End Sub



  
End Class
